@@ -1,6 +1,8 @@
 import logging
-from typing import List
+import time
+from typing import List, Optional
 from abc import ABC, abstractmethod
+from urllib.error import HTTPError
 
 import pandas as pd
 from minio import Minio
@@ -8,6 +10,7 @@ from minio.commonconfig import Tags
 
 from miniops.utils import log_memory_usage
 
+
 class MinioAgent(ABC):
     def __init__(
         self,
@@ -37,14 +40,12 @@ class MinioAgent(ABC):
         # by default this does not do anything, implement your own if needed
         return dataframe
 
-    def process_generator(self, dataframe_generator, preprocessing: bool = True):
+    def process_generator(self, dataframe_generator):
         for counter, dataframe in enumerate(dataframe_generator):
             logging.info(
                 f"processing dataframe #{counter+1}, with {len(dataframe)} items"
             )
             log_memory_usage()
-            if preprocessing:
-                dataframe = self.preprocess(dataframe)
             self.process(dataframe)
 
     def get_object_names_generator_for_objects_in_bucket(self, **kwargs):
@@ -58,16 +59,39 @@ class MinioAgent(ABC):
         )
         return [name for name in names_generator]
 
+    def try_to_read_dataframe_from_minio(
+        self, bucket_name, object_name, max_retries: int = 2, timeout_seconds: int = 10
+    ) -> Optional[pd.DataFrame]:
+        for counter in range(max_retries):
+            try:
+                dataframe = pd.read_json(
+                    f"{self.hostname}/{bucket_name}/{object_name}", lines=True
+                )
+                return dataframe
+            except HTTPError:
+                first_sentence = f"Could not read {object_name} from {bucket_name}, attempt: {counter+1}."
+                if counter < max_retries:
+                    logging.error(
+                        f"{first_sentence} Trying again in {timeout_seconds}..."
+                    )
+                    time.sleep(timeout_seconds)
+                else:
+                    logging.error(f"{first_sentence} Giving up")
+
     def get_df_generator_for_objects_in_bucket(
-        self, names: List[str], bucket_name: str
+        self, names: List[str], bucket_name: str = None
     ):
         if not bucket_name:
             bucket_name = self.data_bucket
         for name in names:
-            dataframe = pd.read_json(
-                f"{self.hostname}/{bucket_name}/{name}", lines=True
+            dataframe = self.try_to_read_dataframe_from_minio(
+                bucket_name=bucket_name, object_name=name
             )
-            yield dataframe
+            if dataframe is not None:
+                dataframe = self.preprocess(dataframe)
+                yield dataframe
+            else:
+                yield None
 
     def has_object_not_been_analysed(self, object_name: str, bucket_name: str) -> bool:
         tags = self.client.get_object_tags(bucket_name, object_name)
